@@ -5,7 +5,7 @@
 % ***
 % =========================================================================
 % ***
-% *** Copyright (c) 2008--2013, Nico Schlömer <nico.schloemer@gmail.com>
+% *** Copyright (c) 2008--2014, Nico Schlömer <nico.schloemer@gmail.com>
 % *** All rights reserved.
 % ***
 % *** Redistribution and use in source and binary forms, with or without
@@ -31,7 +31,7 @@
 % *** POSSIBILITY OF SUCH DAMAGE.
 % ***
 % =========================================================================
-function [desc, extraOpts, funcName, numFunctions] = testfunctions(k)
+function [desc, extraOpts, extraCFOptions, funcName, numFunctions] = testfunctions(k)
 
   % assign the functions to test
   testfunction_handles = {                        ...
@@ -39,6 +39,7 @@ function [desc, extraOpts, funcName, numFunctions] = testfunctions(k)
                            @plain_cos           , ...
                            @sine_with_markers   , ...
                            @markerSizes         , ...
+                           @markerSizes2        , ...
                            @sine_with_annotation, ...
                            @linesWithOutliers   , ...
                            @peaks_contour       , ...
@@ -46,6 +47,7 @@ function [desc, extraOpts, funcName, numFunctions] = testfunctions(k)
                            @peaks_contourf      , ...
                            @many_random_points  , ...
                            @double_colorbar     , ...
+                           @subplot_colorbar    , ...
                            @randomWithLines     , ...
                            @double_axes         , ...
                            @double_axes2        , ...
@@ -125,20 +127,54 @@ function [desc, extraOpts, funcName, numFunctions] = testfunctions(k)
                            @croppedImage        , ...
                            @doubleAxes          , ...
                            @pColorPlot          , ...
-                           @hgTransformPlot
+                           @hgTransformPlot     , ...
+                           @scatter3Plot3       , ...
+                           @scatterPlotMarkers  , ...
+                           @multiplePatches     , ...
+                           @logbaseline         , ...
+                           @alphaImage
                          };
 
   numFunctions = length( testfunction_handles );
 
+  desc = '';
+  funcName = '';
+  extraOpts = {};
+  extraCFOptions = {};
+
+
   if (k<=0)
-      % This is used for querying numFunctions.
-      desc = '';
-      funcName = '';
-      extraOpts = {};
-      return;
+      return;  % This is used for querying numFunctions.
+
   elseif (k<=numFunctions)
-      [desc, extraOpts] = testfunction_handles{k}();
       funcName = func2str(testfunction_handles{k});
+
+      try
+          nargs = nargout(funcName);
+      catch %#ok
+          % In Octave 3.6.4, `nargout` seems not to be able to handle
+          % everything as MATLAB does:
+          %  * it does not support function handles (so using strings)
+          %  * it cannot handle subfunctions apparently, so always use the
+          %    default syntax there
+          %TODO: handle diferent number of arguments in Octave
+          warning('testfunctions:nargoutOctave',...
+              'Cannot determine number of output of "%s". Assuming 2.',funcName)
+          nargs = 2;
+      end
+
+      switch nargs
+          case 3
+              [desc, extraOpts, extraCFOptions] = testfunction_handles{k}();
+
+          case 1
+              desc = testfunction_handles{k}();
+
+          otherwise
+              [desc, extraOpts] = testfunction_handles{k}();
+
+      end
+
   else
       error('testfunctions:outOfBounds', ...
             'Out of bounds (number of testfunctions=%d)', numFunctions);
@@ -147,57 +183,18 @@ function [desc, extraOpts, funcName, numFunctions] = testfunctions(k)
   return;
 end
 % =========================================================================
-% *** FUNCTION one_point
 function [description, extraOpts] = one_point()
 
-%%--------------------------------------------------
-%t_start=0.0;
-%t_end=60;
-%delta_t = 0.1;
-%number_of_intpoints = (t_end - t_start ) / delta_t;
-%
-%xp = linspace(t_start, t_end, number_of_intpoints);
-%yp1 = xp.^2+2;
-%yp2 = xp.^2+50.*sin(xp)-1;
-%yp3 = sin(xp);
-%
-%xp = reshape(xp,size(xp,2),size(xp,1));
-%yp1 = reshape(yp1,size(yp1,2),size(yp1,1));
-%yp2 = reshape(yp2,size(yp2,2),size(yp2,1));
-%yp3 = reshape(yp3,size(yp3,2),size(yp3,1));
-%%--------------------------------------------------
-%
-%% figure
-%% plotyy
-%[AX,P1,P2] = plotyy(xp, [ yp2  , yp1      ], xp     ,   yp3);
-%% setting axis labels
-%set(get(AX(1),'Ylabel'),'String','$E_{kin}$,$E_{pot}$ [J] x $10^{-3}$','Interpreter','tex');
-%set(get(AX(2),'Ylabel'),'String','$E_{pot}$ [J] x $10^{-3}$');
-%xlabel('t [ns]');
-%% line Properties
-%set(P1,'LineWidth',1);
-%set(P1(1),'LineStyle','-','Color',[1 0 0])
-%set(P1(2),'LineStyle','-','Color',[0 0 0]);
-%set(P2,'LineStyle','-','LineWidth',1,'Color',[0 0 1])
-%grid on;
-%hold on;
-%legend1 = legend([P1(1) P1(2) P2],'$E_{tot}$','$E_{kin}$','$E_{pot}$');
-%set(legend1,'Box','off','Orientation','horizontal','Location','NorthOutside');
-%--------------------------------------------------
-% output
-%matlab2tikz('parseStrings', false, 'interpretTickLabelsAsTex', true,'width', '\figurewidth', 'height','\figureheight', 'test.tex')
-
   m = [0 1 1.5 1 -1];
   k = 1:1:length(m);
   plot(k,m,'*-');
-  %plot(1:10)
+
   title({'title', 'multline'})
   %legend(char('Multi-Line Legend Entry','Wont Work 2^2=4'))
   legend('Multi-Line Legend Entry Wont Work 2^2=4')
   xlabel({'one','two','three'});
   ylabel({'one','° ∞', 'three'});
 
-%  plot(0.123, 0.145, 'x');
   set(gca, 'YTick', []);
   set(gca,'XTick',0:1:length(m)-1);
 
@@ -208,7 +205,7 @@ function [description, extraOpts] = one_point()
 
 end
 % =========================================================================
-function [description, extraOpts] = plain_cos()
+function [description, extraOpts, extraOptsCleanFigure] = plain_cos()
 
   fplot( @cos, [0,2*pi] );
 
@@ -232,7 +229,8 @@ function [description, extraOpts] = plain_cos()
   end
 
   description = 'Plain cosine function with minimumPointsDistance of $0.5$.' ;
-  extraOpts = {'minimumPointsDistance', 0.5};
+  extraOpts = {};
+  extraOptsCleanFigure = {'minimumPointsDistance', 0.5};
 
 end
 % =========================================================================
@@ -276,6 +274,31 @@ function [description, extraOpts] = markerSizes()
   description = 'Marker sizes.';
   extraOpts = {};
 end
+
+% =========================================================================
+function [description, extraOpts] = markerSizes2()
+  hold on;
+  grid on;
+  
+  n = 1:10;
+  d = 10;
+  s = round(linspace(6,25,10));
+  e = d * ones(size(n));
+  style = {'bx','rd','go','c.','m+','y*','bs','mv','k^','r<','g>','cp','bh'};
+  nStyles = numel(style);
+
+  for ii = 1:nStyles
+      for jj = 1:10
+        plot(n(jj), ii * e(jj),style{ii},'MarkerSize',s(jj));
+      end
+  end
+  xlim([min(n)-1 max(n)+1]);
+  ylim([0 d*(nStyles+1)]);
+  set(gca,'XTick',n,'XTickLabel',s,'XTickLabelMode','manual');
+
+  description = 'Line plot with with different marker sizes.';
+  extraOpts = {};
+end
 % =========================================================================
 function [description, extraOpts] = sine_with_annotation ()
 
@@ -391,6 +414,22 @@ function [description, extraOpts] = double_colorbar()
 
 end
 % =========================================================================
+function [description, extraOpts] = subplot_colorbar()
+
+  img = rand(100);
+  vec = rand(100,1);
+
+  subplot(2,1,1);
+  imagesc(img,[0 1]);
+  colorbar;
+
+  subplot(2,1,2);
+  plot(vec);
+
+  description = 'Subplot colorbar.';
+  extraOpts = {};
+end
+% =========================================================================
 function [description, extraOpts] = randomWithLines()
 
   X = randn(150,2);
@@ -1314,7 +1353,7 @@ end
 % =========================================================================
 function [description, extraOpts] = scatterPlotRandom()
 
-  n=1:100;
+  n = 1:100;
   scatter(n, n, 1000*rand(length(n),1), n.^8);
   colormap autumn;
   %x = randn( 10, 2 );
@@ -1338,6 +1377,34 @@ function [description, extraOpts] = scatterPlot()
   description = 'Scatter plot with MATLAB(R) data.';
   extraOpts = {};
 
+end
+% =========================================================================
+function [description, extraOpts] = scatterPlotMarkers()
+
+  n = 1:10;
+  d = 10;
+  s = d^2 * n;
+  e = d * ones(size(n));
+  grid on;
+  hold on;
+
+  style = {'bx','rd','go','c.','m+','y*','bs','mv','k^','r<','g>','cp','bh'};
+  names = {'bx','rd','go','c.','m plus','y star','bs','mv',...
+           'k up triangle','r left triangle','g right triangle','cp','bh'};
+
+  nStyles = numel(style);
+  for ii = 1:nStyles
+      scatter(n, ii * e, s, style{ii});
+  end
+  xlim([min(n)-1 max(n)+1]);
+  ylim([0 d*(nStyles+1)]);
+  set(gca,'XTick',n,'XTickLabel',s,'XTickLabelMode','manual');
+
+  legend(names{:});
+
+  description = 'Scatter plot with with different marker sizes and legend.';
+  extraOpts = {};
+
 end
 % =========================================================================
 function [description, extraOpts] = scatter3Plot()
@@ -1383,6 +1450,17 @@ function [description, extraOpts] = scatter3Plot2()
   return
 end
 % =========================================================================
+function [description, extraOpts] = scatter3Plot3()
+  hold on;
+  scatter3(rand(5,1),rand(5,1),rand(5,1),150,...
+           'MarkerEdgeColor','none','MarkerFaceColor','k');
+  scatter3(rand(5,1),rand(5,1),rand(5,1),150,...
+           'MarkerEdgeColor','none','MarkerFaceColor','b');
+
+  description = 'Scatter3 plot with 2 colors (Issue 292)';
+  extraOpts = {};
+end
+% =========================================================================
 function [description, extraOpts] = spherePlot()
 
   sphere(30);
@@ -2149,6 +2227,25 @@ function [description, extraOpts] = pColorPlot()
   extraOpts = {};
 end
 % =========================================================================
+function [description, extraOpts] = multiplePatches()
+
+  xdata = [2     2     0     2     5;
+           2     8     2     4     5;
+           8     8     2     4     8];
+  ydata = [4     4     4     2     0;
+           8     4     6     2     2;
+           4     0     4     0     0];
+  cdata = [15     0     4     6    10;
+           1     2     5     7     9;
+           2     3     0     8     3];
+  p = patch(xdata,ydata,cdata,'Marker','o',...
+            'MarkerFaceColor','flat',...
+            'FaceColor','none');
+
+  description = 'Multiple patches.';
+  extraOpts = {};
+end
+% =========================================================================
 function [description, extraOpts] = hgTransformPlot()
   % Check out
   % http://www.mathworks.de/de/help/matlab/ref/hgtransform.html.
@@ -2180,6 +2277,29 @@ function [description, extraOpts] = hgTransformPlot()
   extraOpts = {};
 end
 % =========================================================================
+function [description, extraOpts] = logbaseline()
+
+  bar([0 1 2], [1 1e-2 1e-5],'basevalue', 1e-6);
+  set(gca,'YScale','log')
+
+  description = 'Logplot with modified baseline.';
+  extraOpts = {};
+end
+% =========================================================================
+function [description, extraOpts] = alphaImage()
+
+  N = 20;
+  h_imsc = imagesc(repmat(1:N, N, 1));
+  mask = zeros(N);
+  mask(N/4:3*N/4, N/4:3*N/4) = 1;
+  set(h_imsc, 'AlphaData', double(~mask));
+  set(h_imsc, 'AlphaDataMapping', 'scaled');
+  set(gca, 'ALim', [-1,1]);
+
+  description = 'Image with alpha channel.';
+  extraOpts = {};
+end
+% =========================================================================
 function env = getEnvironment
   if ~isempty(ver('MATLAB'))
      env = 'MATLAB';
